/*
 * $Revision: 6107 $
 *
 * (C) Copyright 2001 Sun Microsystems, Inc.
 * Copyright (c) OSGi Alliance (2001, 2008). All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package java.math;
public class BigInteger extends java.lang.Number implements java.lang.Comparable {
	public BigInteger(int var0, int var1, java.util.Random var2) { }
	public BigInteger(int var0, java.util.Random var1) { }
	public BigInteger(int var0, byte[] var1) { }
	public BigInteger(java.lang.String var0) { }
	public BigInteger(java.lang.String var0, int var1) { }
	public BigInteger(byte[] var0) { }
	public java.math.BigInteger abs() { return null; }
	public java.math.BigInteger add(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger and(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger andNot(java.math.BigInteger var0) { return null; }
	public int bitCount() { return 0; }
	public int bitLength() { return 0; }
	public java.math.BigInteger clearBit(int var0) { return null; }
	public int compareTo(java.lang.Object var0) { return 0; }
	public int compareTo(java.math.BigInteger var0) { return 0; }
	public java.math.BigInteger divide(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger[] divideAndRemainder(java.math.BigInteger var0) { return null; }
	public double doubleValue() { return 0.0d; }
	public java.math.BigInteger flipBit(int var0) { return null; }
	public float floatValue() { return 0.0f; }
	public java.math.BigInteger gcd(java.math.BigInteger var0) { return null; }
	public int getLowestSetBit() { return 0; }
	public int intValue() { return 0; }
	public boolean isProbablePrime(int var0) { return false; }
	public long longValue() { return 0l; }
	public java.math.BigInteger max(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger min(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger mod(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger modInverse(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger modPow(java.math.BigInteger var0, java.math.BigInteger var1) { return null; }
	public java.math.BigInteger multiply(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger negate() { return null; }
	public java.math.BigInteger not() { return null; }
	public java.math.BigInteger or(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger pow(int var0) { return null; }
	public java.math.BigInteger remainder(java.math.BigInteger var0) { return null; }
	public java.math.BigInteger setBit(int var0) { return null; }
	public java.math.BigInteger shiftLeft(int var0) { return null; }
	public java.math.BigInteger shiftRight(int var0) { return null; }
	public int signum() { return 0; }
	public java.math.BigInteger subtract(java.math.BigInteger var0) { return null; }
	public boolean testBit(int var0) { return false; }
	public byte[] toByteArray() { return null; }
	public java.lang.String toString(int var0) { return null; }
	public static java.math.BigInteger valueOf(long var0) { return null; }
	public java.math.BigInteger xor(java.math.BigInteger var0) { return null; }
	public final static java.math.BigInteger ONE; static { ONE = null; }
	public final static java.math.BigInteger ZERO; static { ZERO = null; }
}

